#include <iostream>
#include <vector>
#include "svg.h"

const auto IMAGE_WIDTH = 400;
const auto IMAGE_HEIGHT = 310;
const auto TEXT_LEFT = 20;
const auto TEXT_BASELINE = 20;
const auto TEXT_WIDTH = 50;
const auto BIN_HEIGHT = 30;
const auto BLOCK_WIDTH = 10;

void
svg_begin(double width, double height)
{
    std::cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    std::cout << "<svg ";
    std::cout << "width='" << width << "' ";
    std::cout << "height='" << height << "' ";
    std::cout << "viewBox='0 0 " << width << " " << height << "' ";
    std::cout << "xmlns='http://www.w3.org/2000/svg'>\n";
    std::cout << "<line x1='10' y1='2' x2='" << width + 10 <<"' y2='2' stroke-dasharray = '10 10' stroke='black'/>\n";
}

void
svg_end(double top)
{
    std::cout << "<line x1='10' y1='" << top << "' x2='" << IMAGE_WIDTH <<"' y2='" << top << "' stroke-dasharray = '10 10' stroke='black'/>\n";
    std::cout << "</svg>\n";
}

void
svg_text(double left, double baseline, std::string text)
{
    std::cout << "<line x1='10' y1='" << baseline -  TEXT_BASELINE << "' x2='10' y2='" << baseline <<"' stroke-dasharray = '10 10' stroke='black'/>\n";
    std::cout << "<text x='" << left << "' y='"<< baseline << "' > " << text << " </text>";
}

void svg_rect(double x, double y, double width, double height)
{
    std::cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << " ' stroke='black' fill='#33A220' ></rect>";
    std::cout << "<line x1='" << IMAGE_WIDTH <<"' y1='" << y << "' x2='" << IMAGE_WIDTH <<"' y2='" << y + height <<"' stroke-dasharray = '10 10' stroke='black'/>\n";
}

    void show_histogram_svg(const std::vector<std::size_t>& bins) {
        double max = 0, scale = 1;
        int maxlen = IMAGE_WIDTH - TEXT_WIDTH;
        for (auto el: bins)
        {
            if (max < el)
            {
                max = el;
            }
        }
        svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);
        double top = 5;
        scale = max * BLOCK_WIDTH / maxlen;
        for (size_t bin : bins)
        {
            const double bin_width = BLOCK_WIDTH * bin;
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bin));
            svg_rect(TEXT_WIDTH, top, bin_width / scale, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
        svg_end(top);
    }
